/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.ArrayList;
import java.util.Collection;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the results of a {@link CollisionCalculation}.
 *
 * @author Gernot Belger
 */
final public class CollisionCalcOverviewResult extends AbstractCalculationExportableResult<CollisionCalculationResults> {

    private static final long serialVersionUID = 1L;

    private static final String JASPER_FILE = "/jasper/templates/sinfo.collision.overview.jrxml";

    public CollisionCalcOverviewResult(final String label, final Collection<ResultRow> rows) {
        super(label, rows);
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results) {
        // do nothing so far
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final RiverInfo river) {
        final int colSize = 3;
        exportContextCSV.writeTitleForTabs("sinfo.export.csv.title.collison.overview", 3); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
                                                                                           // geschrieben wird.
        // Das ist etwas doof.

        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.years));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.collisionCount));

        exportContextCSV.writeCSVLine(header.toArray(new String[colSize]));
    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {
        return formatRow(exportContextCSV, results, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {
        final Collection<String> lines = new ArrayList<>(3);

        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.years));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.collisionCount));
        return lines.toArray(new String[lines.size()]);
    }

    private String[] formatRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(3);

        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.years));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.collisionCount));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    @Override
    protected void addJRTableHeader(final ExportContextCSV exportContextCSV, final MetaAndTableJRDataSource source, final CollisionCalculationResults results) {

        /* column headings */
        exportContextCSV.addJRMetadata(source, "station_header", GeneralResultType.station);
        exportContextCSV.addJRMetadata(source, "collision_years_header", SInfoResultType.years);
        exportContextCSV.addJRMetadata(source, "collision_count_header", SInfoResultType.collisionCount);
    }
}